@@ -35,6 +35,7 @@ import torch
 
 from nemo.core.classes import Loss, typecheck
 from nemo.core.neural_types import LabelsType, LengthsType, LogprobsType, LossType, NeuralType
+from nemo.core.utils.numba_utils import NUMBA_INSTALLATION_MESSAGE
 from nemo.utils import logging, model_utils
 
 try:
@@ -60,15 +61,6 @@ WARP_RNNT_INSTALLATION_MESSAGE = (
     "container that supports RNN-T loss."
 )
 
-NUMBA_INSTALLATION_MESSAGE = (
-    "Could not import `numba`.\n"
-    "To use transducer loss, please install numba in one of the following ways."
-    "1) If using conda, simply install `conda install -c numba numba`\n"
-    "2) If using pip, `pip install --upgrade --ignore-installed numba`\n"
-    "followed by `export NUMBAPRO_LIBDEVICE='/usr/local/cuda/nvvm/libdevice/'` and \n"
-    "`export NUMBAPRO_NVVM='/usr/local/cuda/nvvm/lib64/libnvvm.so'`"
-)
-
 
 @dataclass
 class RNNTLossConfig:
