@@ -15,7 +15,7 @@
 from typing import List
 
 import torch
-from pytorch_lightning.metrics import Metric
+from torchmetrics import Metric
 
 __all__ = ['TopKClassificationAccuracy']
 
