@@ -95,7 +95,7 @@ class QAModel(NLPModel):
         return {'loss': loss, 'lr': lr}
 
     def validation_step(self, batch, batch_idx):
-        if self.testing:
+        if self.trainer.testing:
             prefix = 'test'
         else:
             prefix = 'val'
@@ -117,7 +117,7 @@ class QAModel(NLPModel):
         return self.validation_step(batch, batch_idx)
 
     def validation_epoch_end(self, outputs):
-        if self.testing:
+        if self.trainer.testing:
             prefix = 'test'
         else:
             prefix = 'val'
@@ -158,7 +158,7 @@ class QAModel(NLPModel):
             for u in all_end_logits:
                 end_logits.extend(tensor2list(u))
 
-            eval_dataset = self._test_dl.dataset if self.testing else self._validation_dl.dataset
+            eval_dataset = self._test_dl.dataset if self.trainer.testing else self._validation_dl.dataset
             exact_match, f1, all_predictions, all_nbest = eval_dataset.evaluate(
                 unique_ids=unique_ids,
                 start_logits=start_logits,
