@@ -95,6 +95,9 @@ def get_tokenizer(
     elif tokenizer_name == 'char':
         return CharTokenizer(vocab_file=vocab_file, **special_tokens_dict)
 
+    logging.info(
+        f"Getting HuggingFace AutoTokenizer with pretrained_model_name: {tokenizer_name}, vocab_file: {vocab_file}, special_tokens_dict: {special_tokens_dict}, and use_fast: {use_fast}"
+    )
     return AutoTokenizer(
         pretrained_model_name=tokenizer_name, vocab_file=vocab_file, **special_tokens_dict, use_fast=use_fast
     )
@@ -111,7 +114,7 @@ def get_nmt_tokenizer(
 ):
     """
     Args:
-        model_name: if using a pretrained model from NeMo or HuggingFace
+        model_name: if using a pretrained model from NeMo, HuggingFace, or Megatron
         tokenizer_model: tokenizer model file of sentencepiece or youtokentome
         special_tokens: dict of special tokens
         vocab_file: path to vocab file
@@ -138,7 +141,12 @@ def get_nmt_tokenizer(
         return nemo.collections.common.tokenizers.sentencepiece_tokenizer.SentencePieceTokenizer(
             model_path=tokenizer_model, special_tokens=special_tokens_dict
         )
+    elif library == 'megatron':
+        logging.info(
+            f'Getting Megatron tokenizer for pretrained model name: {model_name} and custom vocab file: {vocab_file}'
+        )
+        return get_tokenizer(tokenizer_name=model_name, vocab_file=vocab_file)
     else:
         raise NotImplementedError(
-            'Currently we only support "yttm", "huggingface", and "sentencepiece" tokenizer library.'
+            'Currently we only support "yttm", "huggingface", "megatron", and "sentencepiece" tokenizer library.'
         )
