@@ -19,6 +19,7 @@ import librosa
 import matplotlib.pylab as plt
 import numpy as np
 import torch
+from numba import jit, prange
 from numpy import ndarray
 from pesq import pesq
 from pystoi import stoi
@@ -35,6 +36,40 @@ class OperationMode(Enum):
     infer = 2
 
 
+def binarize_attention(attn, in_len, out_len):
+    """Convert soft attention matrix to hard attention matrix.
+
+    Args:
+        attn (torch.Tensor): B x 1 x max_mel_len x max_text_len. Soft attention matrix.
+        in_len (torch.Tensor): B. Lengths of texts.
+        out_len (torch.Tensor): B. Lengths of spectrograms.
+
+    Output:
+        attn_out (torch.Tensor): B x 1 x max_mel_len x max_text_len. Hard attention matrix, final dim max_text_len should sum to 1.
+    """
+    b_size = attn.shape[0]
+    with torch.no_grad():
+        attn_cpu = attn.data.cpu().numpy()
+        attn_out = torch.zeros_like(attn)
+        for ind in range(b_size):
+            hard_attn = mas(attn_cpu[ind, 0, : out_len[ind], : in_len[ind]])
+            attn_out[ind, 0, : out_len[ind], : in_len[ind]] = torch.tensor(hard_attn, device=attn.device)
+    return attn_out
+
+
+def binarize_attention_parallel(attn, in_lens, out_lens):
+    """For training purposes only. Binarizes attention with MAS.
+           These will no longer recieve a gradient.
+
+        Args:
+            attn: B x 1 x max_mel_len x max_text_len
+        """
+    with torch.no_grad():
+        attn_cpu = attn.data.cpu().numpy()
+        attn_out = b_mas(attn_cpu, in_lens.cpu().numpy(), out_lens.cpu().numpy(), width=1)
+    return torch.from_numpy(attn_out).to(attn.get_device())
+
+
 def get_mask_from_lengths(lengths, max_len=None):
     if not max_len:
         max_len = torch.max(lengths).item()
@@ -43,6 +78,79 @@ def get_mask_from_lengths(lengths, max_len=None):
     return mask
 
 
+@jit(nopython=True)
+def mas(attn_map, width=1):
+    # assumes mel x text
+    opt = np.zeros_like(attn_map)
+    attn_map = np.log(attn_map)
+    attn_map[0, 1:] = -np.inf
+    log_p = np.zeros_like(attn_map)
+    log_p[0, :] = attn_map[0, :]
+    prev_ind = np.zeros_like(attn_map, dtype=np.int64)
+    for i in range(1, attn_map.shape[0]):
+        for j in range(attn_map.shape[1]):  # for each text dim
+            prev_j = np.arange(max(0, j - width), j + 1)
+            prev_log = np.array([log_p[i - 1, prev_idx] for prev_idx in prev_j])
+
+            ind = np.argmax(prev_log)
+            log_p[i, j] = attn_map[i, j] + prev_log[ind]
+            prev_ind[i, j] = prev_j[ind]
+
+    # now backtrack
+    curr_text_idx = attn_map.shape[1] - 1
+    for i in range(attn_map.shape[0] - 1, -1, -1):
+        opt[i, curr_text_idx] = 1
+        curr_text_idx = prev_ind[i, curr_text_idx]
+    opt[0, curr_text_idx] = 1
+
+    assert opt.sum(0).all()
+    assert opt.sum(1).all()
+
+    return opt
+
+
+@jit(nopython=True)
+def mas_width1(attn_map):
+    """mas with hardcoded width=1"""
+    # assumes mel x text
+    opt = np.zeros_like(attn_map)
+    attn_map = np.log(attn_map)
+    attn_map[0, 1:] = -np.inf
+    log_p = np.zeros_like(attn_map)
+    log_p[0, :] = attn_map[0, :]
+    prev_ind = np.zeros_like(attn_map, dtype=np.int64)
+    for i in range(1, attn_map.shape[0]):
+        for j in range(attn_map.shape[1]):  # for each text dim
+            prev_log = log_p[i - 1, j]
+            prev_j = j
+
+            if j - 1 >= 0 and log_p[i - 1, j - 1] >= log_p[i - 1, j]:
+                prev_log = log_p[i - 1, j - 1]
+                prev_j = j - 1
+
+            log_p[i, j] = attn_map[i, j] + prev_log
+            prev_ind[i, j] = prev_j
+
+    # now backtrack
+    curr_text_idx = attn_map.shape[1] - 1
+    for i in range(attn_map.shape[0] - 1, -1, -1):
+        opt[i, curr_text_idx] = 1
+        curr_text_idx = prev_ind[i, curr_text_idx]
+    opt[0, curr_text_idx] = 1
+    return opt
+
+
+@jit(nopython=True, parallel=True)
+def b_mas(b_attn_map, in_lens, out_lens, width=1):
+    assert width == 1
+    attn_out = np.zeros_like(b_attn_map)
+
+    for b in prange(b_attn_map.shape[0]):
+        out = mas_width1(b_attn_map[b, 0, : out_lens[b], : in_lens[b]])
+        attn_out[b, 0, : out_lens[b], : in_lens[b]] = out
+    return attn_out
+
+
 def griffin_lim(magnitudes, n_iters=50, n_fft=1024):
     """
     Griffin-Lim algorithm to convert magnitude spectrograms to audio signals
