@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from nemo.collections.tts.models.aligner import AlignerModel
 from nemo.collections.tts.models.degli import DegliModel
 from nemo.collections.tts.models.ed_mel2spec import EDMel2SpecModel
 from nemo.collections.tts.models.fastpitch import FastPitchModel
@@ -48,4 +49,5 @@ __all__ = [
     "FastSpeech2Model",
     "FastPitchHifiGanE2EModel",
     "FastSpeech2HifiGanE2EModel",
+    "AlignerModel",
 ]
