@@ -337,7 +337,8 @@ class HifiGanModel(Vocoder, Exportable):
         Override this method to prepare module for export. This is in-place operation.
         Base version does common necessary module replacements (Apex etc)
         """
-        self.generator.remove_weight_norm()
+        if self.generator is not None:
+            self.generator.remove_weight_norm()
 
     def input_example(self):
         """
@@ -348,3 +349,9 @@ class HifiGanModel(Vocoder, Exportable):
         par = next(self.parameters())
         mel = torch.randn((1, self.cfg['preprocessor']['nfilt'], 96), device=par.device, dtype=par.dtype)
         return ({'spec': mel},)
+
+    def forward_for_export(self, spec):
+        """
+        Runs the generator, for inputs and outputs see input_types, and output_types
+        """
+        return self.generator(x=spec)
