@@ -82,12 +82,13 @@ class TrainerConfig:
     prepare_data_per_node: bool = True
     amp_backend: str = 'native'
     amp_level: str = 'O2'  # backward compatible, todo: remove in v1.0.0
-    enable_pl_optimizer: Optional[bool] = None
     plugins: Optional[Any] = None  # Optional[Union[str, list]]
     move_metrics_to_cpu: bool = False
     multiple_trainloader_mode: str = 'max_size_cycle'
     limit_predict_batches: float = 1.0
     stochastic_weight_avg: bool = False
+    gradient_clip_algorithm: str = 'norm'
+    max_time: Optional[Any] = None  # can be one of Union[str, timedelta, Dict[str, int], None]
 
 
 # Register the trainer config.
